﻿using System.Collections.Generic;
using System.Text;
using PAT.Common.Classes.Assertion;
using PAT.Common.Classes.Expressions;
using PAT.Common.Classes.Expressions.ExpressionClass;
using PAT.Common.Classes.Ultility;
using PAT.SCSP.Assertions;

namespace PAT.SCSP.LTS{
    public sealed class Interleave : Process
    {
        public Process FirstProcess;
        public Process SecondProcess;

        public Interleave(Process first, Process second)
        {
            FirstProcess = first;
            SecondProcess = second;
            ProcessID = FirstProcess.ToString() + "|||" + SecondProcess.ToString();
            ProcessID = DataStore.DataManager.InitializeProcessID(ProcessID);
        }

        public override void MoveOneStep(Valuation GlobalEnv, List<Configuration> list)
        {
            System.Diagnostics.Debug.Assert(list.Count == 0);

            bool allTerminationCount = true;
            bool hasAtomicTermination = false;

            for (int i = 0; i < 2; i++)
            {
                Process process;

                if (i == 0)
                    process = FirstProcess;
                else
                {
                    process = SecondProcess;
                }

                List<Configuration> list1 = new List<Configuration>();
                process.MoveOneStep(GlobalEnv, list1);
                bool hasTermination = false;

                for (int j = 0; j < list1.Count; j++)
                {
                    Configuration step = list1[j];

                    if (step.Event == Constants.TERMINATION)
                    {
                        hasTermination = true;

                        if (step.IsAtomic)
                        {
                            hasAtomicTermination = true;
                        } 
                    }
                    else
                    {
                        if (AssertionBase.CalculateParticipatingProcess)
                        {
                            step.ParticipatingProcesses = new string[] {i.ToString()};
                        }

                        Interleave interleave;

                        if(i == 0)
                        {
                            interleave = new Interleave(step.Process, SecondProcess);
                        }
                        else
                        {
                            interleave = new Interleave(FirstProcess, step.Process);
                        }

                        step.Process = interleave;
                        list.Add(step);
                    }
                }

                ////to check whether there are synchoronous channel input/output
                //if (Specification.HasSyncrhonousChannel)
                //{
                //    SynchronousChannelInputOutput(list, i, GlobalEnv, null);
                //}

                if (!hasTermination)
                {
                    allTerminationCount = false;
                }
            }

            if (allTerminationCount)
            {
                Configuration temp = new Configuration(new Stop(), Constants.TERMINATION, GlobalEnv, false);

                if (hasAtomicTermination)
                {
                    temp.IsAtomic = true;
                }

                if (AssertionBase.CalculateParticipatingProcess)
                {
                    temp.ParticipatingProcesses = new string[2];
                    for (int i = 0; i < 2; i++)
                    {
                        temp.ParticipatingProcesses[i] = i.ToString();
                    }  
                }
                list.Add(temp);
            }
            
            //return returnList;
        }

        //private void SynchronousChannelInputOutput(List<Configuration> returnList, int i, Valuation GlobalEnv, string evt)
        //{
        //    List<ConfigurationWithChannelData> outputs = new List<ConfigurationWithChannelData>();
        //    Processes[i].SyncOutput(GlobalEnv, outputs);

        //    foreach (ConfigurationWithChannelData pair in outputs)
        //    {
        //        Configuration vm = pair;
 
        //        if(evt != null && vm.Event != evt)
        //        {
        //            continue;    
        //        }
                
        //        Process output = vm.Process;

        //        for (int k = 0; k < Processes.Count; k++)
        //        {
        //            if (k != i)
        //            {
        //                List<Configuration> syncedProcess = new List<Configuration>();
        //                Processes[k].SyncInput(pair, syncedProcess);

        //                foreach (Configuration p in syncedProcess)
        //                {
        //                    List<Process> newProcess = new List<Process>(Processes.Count);
        //                    newProcess.AddRange(Processes);
        //                    newProcess[i] = output;
        //                    newProcess[k] = p.Process;
                            
        //                    Interleave interleave = new Interleave(newProcess);
        //                    Configuration newStep = new Configuration(interleave, vm.Event, vm.HidenEvent, GlobalEnv, false);
        //                    newStep.IsAtomic = vm.IsAtomic || p.IsAtomic;

        //                    if (AssertionBase.CalculateParticipatingProcess)
        //                    {
        //                        newStep.ParticipatingProcesses = new string[]{i.ToString(), k.ToString()};
        //                    }

        //                    returnList.Add(newStep);
        //                }
        //            }
        //        }
        //    }
        //}

        //public override void SyncOutput(Valuation GlobalEnv, List<ConfigurationWithChannelData> list)
        //{
        //    //List<ConfigurationWithChannelData> returnList = new List<ConfigurationWithChannelData>();
            
        //    for (int i = 0; i < Processes.Count; i++)
        //    {
        //        Process process = Processes[i];
        //        List<ConfigurationWithChannelData> list1 = new List<ConfigurationWithChannelData>();
        //        process.SyncOutput(GlobalEnv, list1);

        //        foreach (ConfigurationWithChannelData pair in list1)
        //        {
        //            Configuration step = pair;

        //            List<Process> newProcess = new List<Process>(Processes.Count);
        //            newProcess.AddRange(Processes);
        //            newProcess[i] = step.Process;

        //            step.Process = new Interleave(newProcess);
        //        }

        //        list.AddRange(list1);
        //    }

        //    //return returnList;
        //}

        //public override void SyncInput(ConfigurationWithChannelData eStep, List<Configuration> list)
        //{
        //    //List<Configuration> returnList = new List<Configuration>();
            
        //    for (int i = 0; i < Processes.Count; i++)
        //    {
        //        Process process = Processes[i];
        //        List<Configuration> list1 = new List<Configuration>();
        //        process.SyncInput(eStep, list1);

        //        for (int j = 0; j < list1.Count; j++)
        //        {
        //            Configuration step = list1[j];

        //            List<Process> newProcess = new List<Process>(Processes.Count);
        //            newProcess.AddRange(Processes);
        //            newProcess[i] = step.Process;
        //            step.Process = new Interleave(newProcess);
        //            list.Add(step);
        //        }
        //    }

        //    //return returnList;
        //}


        //public override List<string> GetFairEvents(Configuration eStep, bool isWeak, bool isLive)
        //{
        //    List<string> resultList = new List<string>();
        //    foreach (Process process in Processes)
        //    {
        //        Common.Classes.Ultility.Ultility.AddList(resultList, process.GetFairEvents(eStep, isWeak, isLive));
        //    }
        //    return resultList;
        //}

        //public override HashSet<string> GetAlphabets(Dictionary<string, string> visitedDefinitionRefs)
        //{
        //    if (Processes == null)
        //    {
        //        return IndexedProcessDefinition.Process.GetAlphabets(visitedDefinitionRefs);
        //    }

        //    HashSet<string> toReturn = new HashSet<string>();

        //    for (int i = 0; i < Processes.Count; i++)
        //    {
        //        Process process = Processes[i];
        //        toReturn.UnionWith(process.GetAlphabets(visitedDefinitionRefs));
        //    }

        //    return toReturn;
        //}

        //public override List<string> GetGlobalVariables()
        //{
        //    if (Processes == null)
        //    {
        //        return IndexedProcessDefinition.Process.GetGlobalVariables();
        //    }

        //    List<string> Variables = new List<string>();

        //    foreach (Process var in Processes)
        //    {
        //        Common.Classes.Ultility.Ultility.AddList(Variables, var.GetGlobalVariables());
        //    }

        //    return Variables;
        //}

        //public override List<string> GetChannels()
        //{
        //    if (Processes == null)
        //    {
        //        return IndexedProcessDefinition.Process.GetChannels();
        //    }

        //    List<string> channels = new List<string>();

        //    foreach (Process var in Processes)
        //    {
        //        Common.Classes.Ultility.Ultility.AddList(channels, var.GetChannels());
        //    }

        //    return channels;
        //}

        public override string ToString()
        {
           return FirstProcess.ToString() + "|||" + SecondProcess.ToString();
        }

        //public override Process ClearConstant(Dictionary<string, Expression> constMapping)
        //{
        //    List<Process> newProceses = new List<Process>();

        //    if (Specification.IsParsing)
        //    {
        //        if (Processes == null)
        //        {
        //            return new Interleave(IndexedProcessDefinition.ClearConstant(constMapping));
        //        }

        //        foreach (Process procese in Processes)
        //        {
        //            newProceses.Add(procese.ClearConstant(constMapping));
        //        }

        //        return new Interleave(newProceses);
        //    }

        //    List<Process> newnewProcesses = Processes;

        //    if(Processes == null)
        //    {
        //        //return new IndexInterleave(IndexedProcessDefinition.GetIndexedProcesses(constMapping));                    
        //        newnewProcesses = IndexedProcessDefinition.GetIndexedProcesses(constMapping);
        //    }

        //    int size = newnewProcesses.Count;
        //    Dictionary<string, int> processCounters = new Dictionary<string, int>(size);

        //    for (int i = 0; i < size; i++)
        //    {
        //        Process newProc = newnewProcesses[i].ClearConstant(constMapping);

        //        if (newProc is Interleave)
        //        {
        //            List<Process> processes = (newProc as Interleave).Processes;

        //            foreach (var processe in processes)
        //            {
        //                string tmp = processe.ProcessID;
        //                if (!processCounters.ContainsKey(tmp))
        //                {
        //                    newProceses.Add(processe);
        //                    processCounters.Add(tmp, 1);
        //                }
        //                else
        //                {
        //                    processCounters[tmp] = processCounters[tmp] + 1;
        //                }
        //            }
        //        }
        //        else if (newProc is IndexInterleaveAbstract)
        //        {
        //            IndexInterleaveAbstract intAbs = newProc as IndexInterleaveAbstract;

        //            foreach (var processe in intAbs.Processes)
        //            {
        //                string tmp = processe.ProcessID;
        //                if (!processCounters.ContainsKey(tmp))
        //                {
        //                    newProceses.Add(processe);
        //                    processCounters.Add(tmp, intAbs.ProcessesActualSize[tmp]);
        //                }
        //                else
        //                {
        //                    processCounters[tmp] = processCounters[tmp] + intAbs.ProcessesCounter[tmp];
        //                }
        //            }
        //        }
        //        else
        //        {
        //            if (!processCounters.ContainsKey(newProc.ProcessID))
        //            {
        //                newProceses.Add(newProc);
        //                processCounters.Add(newProc.ProcessID, 1);
        //            }
        //            else
        //            {
        //                processCounters[newProc.ProcessID] = processCounters[newProc.ProcessID] + 1;
        //            }
        //        }
        //    }

        //    foreach (KeyValuePair<string, int> pair in processCounters)
        //    {
        //        if (pair.Value > 1 || pair.Value == -1)
        //        {
        //            IndexInterleaveAbstract toReturn = new IndexInterleaveAbstract(newProceses, processCounters);
        //            toReturn.ProcessesActualSize = new Dictionary<string, int>(processCounters);
        //            return toReturn;
        //        }
        //    }

        //    return new Interleave(newProceses);
        //}

        public override bool MustBeAbstracted()
        {
            return false;
        }

        public override bool IsBDDEncodable()
        {
            return true;
        }

        public override Process GetTopLevelConcurrency(List<string> visitedDef)
        {
            return this;
        }
    }
}