tree grammar SCSPTreeWalker;

options
{
	tokenVocab=SCSPTree;
	ASTLabelType = CommonTree;
	language=CSharp2; 
	//backtrack = true;
	memoize=true;
}

@namespace	
{PAT.SCSP.LTS}

@header{
using System.Collections.Generic;
using System.IO;
using PAT.Common;
using PAT.Common.Classes.Assertion;
using PAT.SCSP.Assertions;
}

@members
{	
	public static Stack<String> paraphrases = new Stack<String>();   
	private List<DefinitionRef> defList = new List<DefinitionRef>();  
			
	private string options=""; 
	public Specification Spec;
	 
	public override String GetErrorMessage(RecognitionException e, String[] tokenNames)
	{
    		string msg = null;
    		if ( e is NoViableAltException ) {    		
			msg = "Invalid Symbol: '" + e.Token.Text + "'";
    		}
    		else 
    		{
    			msg = base.GetErrorMessage(e, tokenNames);    		
    		}

        	if (paraphrases.Count > 0)
        	{
            		string paraphrase = (string)paraphrases.Peek();
            		msg = msg + " " + paraphrase;
        	}
        	
        	msg = msg + "\n" + "Line " + e.Token.Line + ", Column " + e.Token.CharPositionInLine;
        	
        	return msg;
        }
        
        public void program(string option){
        	options = option;
        	specDef();
        	foreach(var dref in defList){
        		dref.Def = Spec.DefinitionDatabase[dref.Name];
        	}
        }
}

@rulecatch {
catch(RecognitionException re){
	string sError=GetErrorMessage(re, tokenNames);
	throw new ParsingException(sError, re.Token);
} 
}

specDef	
@init  { paraphrases.Push(" SCSP input analysis");  }
@after { paraphrases.Pop(); }
:

	  ^(SPEC_NODE (p=process{Spec.DefinitionDatabase.Add(p.Name, p);})+  
	  		(a=assertion)+
	  )
;

process returns[Definition pDef]
@init  { paraphrases.Push(" process equation analysis");  }
@after { paraphrases.Pop(); }
	:
	^(PROCESS_NODE v=VAR p=processDef{pDef= new Definition($v.Text, p);})
	;
	
processDef returns[Process proc]
@init  { paraphrases.Push(" process definition analysis");  }
@after { paraphrases.Pop(); }
	:
	  ^(PROC_DEF_NODE v=VAR p=processDef{proc = new EventPrefix($v.Text, p);})
	| ^(PROC_DEF_NODE v1=VAR v2=VAR{
		DefinitionRef p1 = new DefinitionRef($v1.Text);
		defList.Add(p1);
		DefinitionRef p2 = new DefinitionRef($v2.Text);
		defList.Add(p2);		
		proc = new Interleave(p1, p2);
		})
	| ^(PROC_DEF_NODE s=SKIP{proc = new Skip();})
	| ^(PROC_DEF_NODE v3=VAR{DefinitionRef pref = new DefinitionRef(v3.Text); 
		defList.Add(pref);
		proc = pref;})
	;
	
assertion
@init{AssertionBase ass;
string assString;}
	:
	  ^(DEADLOCK_NODE  v=VAR{
	  var dref1 = new DefinitionRef($v.Text);
	  defList.Add(dref1);
	  ass = new SCSPAssertionDeadLock(dref1); ass.AssertToken = $v.Token;
	assString = ass.ToString();
	if(Spec.AssertionDatabase.ContainsKey(assString))
	{
		throw new ParsingException("Assertion " + assString + " is defined already!", ass.AssertToken);		      
	}else{
		Spec.AssertionDatabase.Add(assString, ass);		      
	}
	  })
	| ^(REACH_NODE v1=VAR v2=VAR{
	var dref2 = new DefinitionRef($v1.Text);
	defList.Add(dref2);
	ass = new SCSPAssertionReachability(dref2, $v2.Text); ass.AssertToken = $v1.Token;
	assString = ass.ToString();
	if(Spec.AssertionDatabase.ContainsKey(assString))
	{
		throw new ParsingException("Assertion " + assString + " is defined already!", ass.AssertToken);		      
	}else{
		Spec.AssertionDatabase.Add(assString, ass);		      
	}
	})
	;